#line 2077 "ASTGRC.nw"
#ifndef _ASTGRC_HPP
#  define _ASTGRC_HPP

#  include "AST.hpp"
#  include <assert.h>
#  include <stack>
#  include <map>
#  include <set>

namespace ASTGRC {
  using namespace IR;
  using namespace AST;
  using std::map;
  using std::set;

  class GrcSynth;

  
#line 27 "ASTGRC.nw"
class CompletionCodes : public Visitor {
  int maxOverModule; // Maximum code for this
  map<Abort*, int> codeOfAbort; // Code for each weak abort
  map<SignalSymbol*, int> codeOfTrapSymbol; // Code for each trap symbol
  map<Trap*, int> codeOfTrap; // Code for each trap statement
public:

  void alsoMax(AST::ASTNode *n, int &m) {
     int max = recurse(n);
     if (max > m) m = max;
  }

  int recurse(AST::ASTNode *n) {
     if (n) return n->welcome(*this).i;
     else return 0;
  }

  
#line 51 "ASTGRC.nw"
CompletionCodes(Module *m)
{
  assert(m);
  assert(m->body);
  maxOverModule = recurse(m->body);
  if (maxOverModule <= 1) maxOverModule = 1;
}

virtual ~CompletionCodes() {}
#line 65 "ASTGRC.nw"
  int max() const { return maxOverModule; }

  int operator [] (Abort *a) {
    assert(codeOfAbort.find(a) != codeOfAbort.end());
    return codeOfAbort[a];
  }

  int operator [] (SignalSymbol *ts) {
    assert(codeOfTrapSymbol.find(ts) != codeOfTrapSymbol.end());
    return codeOfTrapSymbol[ts];
  }

  int operator [] (Trap *t) {
    assert(codeOfTrap.find(t) != codeOfTrap.end());
    return codeOfTrap[t];
  }
#line 86 "ASTGRC.nw"
Status visit(Signal &s) { return recurse(s.body); }
Status visit(Var &s) { return recurse(s.body); }
Status visit(Loop &s) { return recurse(s.body); }
Status visit(Repeat &s) { return recurse(s.body); }
Status visit(Suspend &s) { return recurse(s.body); }
Status visit(PredicatedStatement &s) { return recurse(s.body); }
#line 95 "ASTGRC.nw"
Status visit(StatementList &l) {
  int max = 1;
  for (vector<Statement*>::iterator i = l.statements.begin() ;
       i != l.statements.end() ; i++ ) alsoMax(*i, max);
  return max;
}
#line 104 "ASTGRC.nw"
Status visit(ParallelStatementList &l) {
  int max = 1;
  for (vector<Statement*>::iterator i = l.threads.begin() ;
       i != l.threads.end() ; i++ ) alsoMax(*i, max);
  return max;
}
#line 113 "ASTGRC.nw"
Status visit(IfThenElse& n) {
  int max = 1;
  alsoMax(n.then_part, max);
  alsoMax(n.else_part, max);
  return max;
}
#line 126 "ASTGRC.nw"
Status visit(Emit&) { return Status(0); }
Status visit(Assign&) { return Status(0); }
Status visit(ProcedureCall&) { return Status(0); }
Status visit(TaskCall&) { return Status(0); }
Status visit(Exec&) { return Status(0); }
Status visit(Exit&) { return Status(0); }
Status visit(Run&) { return Status(0); }
Status visit(Pause&) { return Status(0); }
#line 142 "ASTGRC.nw"
Status visit(Abort &s) {
  int max = 1;
  alsoMax(s.body, max);
  for (vector<PredicatedStatement*>::iterator i = s.cases.begin() ;
       i != s.cases.end() ; i++ ) alsoMax(*i, max);
  if (s.is_weak) {
    int code = max + 1;
    codeOfAbort[&s] = code;
    assert(code >= 2);
    max += 1 + s.cases.size();
  }
  return max;
}
#line 162 "ASTGRC.nw"
Status visit(Trap &s) {
  int max = 1;
  alsoMax(s.body, max);

  // FIXME: is this the right order?  Should the predicates be
  // considered before or after the code is assigned?
 
  for (vector<PredicatedStatement*>::iterator i = s.handlers.begin() ;
       i != s.handlers.end() ; i++ ) alsoMax(*i, max);

  max++; // Allocate an exit level for this trap statement

  codeOfTrap[&s] = max;

  assert(s.symbols);
  for (SymbolTable::const_iterator i = s.symbols->begin() ; i !=
       s.symbols->end() ; i++) {
    SignalSymbol *ts = dynamic_cast<SignalSymbol*>(*i);
    assert(ts);
    assert(ts->kind == SignalSymbol::Trap);
    codeOfTrapSymbol[ts] = max;
  }

  return max;
}
#line 45 "ASTGRC.nw"
};

#line 2096 "ASTGRC.nw"
  
#line 208 "ASTGRC.nw"
class Cloner : public Visitor {
public:
  template <class T> T* operator() (T* n) {
    if (!n) return NULL;
    T* result = dynamic_cast<T*>(n->welcome(*this).n);
    assert(result);
    return result;
  }

  
#line 238 "ASTGRC.nw"
SignalSymbol *cloneLocalSignal(SignalSymbol *s, SymbolTable *st) {
  assert(s);
  assert(newsig.find(s) == newsig.end()); // Should not already be there
  assert(st);

  string name = s->name;
  int next = 1;
  while (st->contains(name)) {
    char buf[10];
    sprintf(buf, "%d", next++);
    name = s->name + '_' + buf;
  }
  SignalSymbol::kinds kind =
    (s->kind == SignalSymbol::Trap) ? SignalSymbol::Trap : SignalSymbol::Local;
  SignalSymbol *result =
    new SignalSymbol(name, s->type, kind, clone(s->combine),
                     clone(s->initializer), clone(s->presence),
                     clone(s->value));
  st->enter(result);
  newsig[s] = result;
  return result;
}
#line 265 "ASTGRC.nw"
void sameSig(SignalSymbol *s) {
  assert(s);
  assert(newsig.find(s) == newsig.end());
  newsig[s] = s;
}
#line 275 "ASTGRC.nw"
void clearSig(SignalSymbol *orig) {
  assert(orig);
  map<SignalSymbol*, SignalSymbol*>::iterator i = newsig.find(orig);
  assert(i != newsig.end());
  newsig.erase(i);
}

#line 219 "ASTGRC.nw"
  virtual ~Cloner() {}

protected:
  template <class T> T* clone(T* n) { return (*this)(n); }

  
#line 231 "ASTGRC.nw"
map<SignalSymbol*, SignalSymbol*> newsig;
#line 286 "ASTGRC.nw"
Status visit(Emit &s) {
  return new Emit(clone(s.signal), clone(s.value));
}

Status visit(Exit &s) {
  return new Exit(clone(s.trap), clone(s.value));
}

Status visit(Assign &s) {
  return new Assign(clone(s.variable), clone(s.value));
}
#line 302 "ASTGRC.nw"
Status visit(Literal &s) { return new Literal(s.value, s.type); }

Status visit(LoadVariableExpression &s) {
  return new LoadVariableExpression(clone(s.variable));
}

Status visit(LoadSignalExpression &s) {
  return new LoadSignalExpression(s.type, clone(s.signal));
}

Status visit(LoadSignalValueExpression &s) {
  return new LoadSignalValueExpression(clone(s.signal));
}
#line 320 "ASTGRC.nw"
Status visit(UnaryOp &s) {
  return new UnaryOp(s.type, s.op, clone(s.source));
}

Status visit(BinaryOp &s) {
  return new BinaryOp(s.type, s.op, clone(s.source1), clone(s.source2));
}
#line 332 "ASTGRC.nw"
Status visit(FunctionCall &s) {
  FunctionCall *c = new FunctionCall(clone(s.callee));
  for (vector<Expression*>::const_iterator i = s.arguments.begin() ;
       i != s.arguments.end() ; i++) {
    assert(*i);
    c->arguments.push_back(clone(*i));
  }
  return c;
}
#line 346 "ASTGRC.nw"
Status visit(CheckCounter &s) {
  return new CheckCounter(s.type, clone(s.counter));
}
#line 355 "ASTGRC.nw"
Status visit(Counter &s) { return &s; }
#line 363 "ASTGRC.nw"
Status visit(VariableSymbol &s) { return &s; }
Status visit(ConstantSymbol &s) { return &s; }
Status visit(BuiltinConstantSymbol &s) { return &s; }
Status visit(BuiltinSignalSymbol &s) { return &s; }
Status visit(FunctionSymbol &s) { return &s; }
Status visit(BuiltinFunctionSymbol &s) { return &s; }
#line 380 "ASTGRC.nw"
Status visit(SignalSymbol &s) {
  assert(newsig.find(&s) != newsig.end()); // should be there
  return newsig[&s];
}
#line 225 "ASTGRC.nw"
};

#line 2098 "ASTGRC.nw"
  
#line 425 "ASTGRC.nw"
struct Context {
  int size;
  std::stack<GRCNode**> continuations;

  Context(int sz) : size(sz) {
    assert(sz >= 2); // Must at least have termination at levels 0 and 1
    continuations.push(new GRCNode*[size]);
    for (int i = 0 ; i < size ; i++ ) continuations.top()[i] = 0;
  }
  ~Context() {}

  void push(Context &c) {
    GRCNode **parent = c.continuations.top();
    continuations.push(new GRCNode*[size]);
    GRCNode **child = continuations.top();
    for ( int i = 0 ; i < size ; i++ ) child[i] = parent[i];
  }

  void push() { push(*this); }

  void pop() {
    delete [] continuations.top();
    continuations.pop();
  }

  GRCNode *& operator ()(int k) {
    assert(k >= 0);
    assert(k < size);
    return continuations.top()[k];
  }
};
#line 2099 "ASTGRC.nw"
  
#line 599 "ASTGRC.nw"
class GrcWalker : public Visitor {
protected:
  Context &context;
  GrcSynth &environment;
  Cloner &clone;
public:
  GrcWalker(Context &, GrcSynth &);

  GRCNode *synthesize(ASTNode *n) {
    assert(n);
    n->welcome(*this);
    assert(context(0));
    return context(0);
  }

  GRCNode *recurse(ASTNode *n) {
    context.push();
    GRCNode *nn = synthesize(n);
    context.pop();
    return nn;
  }

  static GRCNode* push_onto(GRCNode *& b, GRCNode* n) {
    *n >> b;
    b = n;
    return b;
  }

  STNode *stnode(const ASTNode &);
};
#line 2100 "ASTGRC.nw"
  
#line 674 "ASTGRC.nw"
class Surface : public GrcWalker {
public:
  Surface(Context &c, GrcSynth &e) : GrcWalker(c, e) {}
  
#line 716 "ASTGRC.nw"
Status visit(Pause &);
#line 755 "ASTGRC.nw"
Status visit(Exit &);
#line 782 "ASTGRC.nw"
Status visit(Emit &s) {
  push_onto(context(0), new Action(clone(&s)));
  return Status();
}
#line 805 "ASTGRC.nw"
Status visit(Assign &s) {
  Action *a = new Action(clone(&s));
  *a >> context(0);
  context(0) = a;
  return Status();
}
#line 840 "ASTGRC.nw"
Status visit(IfThenElse &);
#line 901 "ASTGRC.nw"
Status visit(StatementList &);
#line 966 "ASTGRC.nw"
Status visit(Loop &);
#line 1013 "ASTGRC.nw"
Status visit(Repeat &);
#line 1187 "ASTGRC.nw"
Status visit(Suspend &);
#line 1387 "ASTGRC.nw"
Status visit(Abort &);
#line 1424 "ASTGRC.nw"
Status visit(ParallelStatementList &);
#line 1731 "ASTGRC.nw"
Status visit(Trap &);
#line 1800 "ASTGRC.nw"
Status visit(Signal &);
#line 1839 "ASTGRC.nw"
Status visit(Var &);
#line 1855 "ASTGRC.nw"
Status visit(Exec &) { return Status(); }
#line 1869 "ASTGRC.nw"
Status visit(ProcedureCall &) { return Status(); }
#line 678 "ASTGRC.nw"
};
#line 2101 "ASTGRC.nw"
  
#line 682 "ASTGRC.nw"
class Depth : public GrcWalker {
public:
  Depth(Context &c, GrcSynth &e) : GrcWalker(c, e) {}
  
#line 732 "ASTGRC.nw"
Status visit(Pause &);
#line 768 "ASTGRC.nw"
Status visit(Exit &) { return Status(); }
#line 789 "ASTGRC.nw"
Status visit(Emit &) { return Status(); }
#line 814 "ASTGRC.nw"
Status visit(Assign &) { return Status(); }
#line 861 "ASTGRC.nw"
Status visit(IfThenElse &);
#line 919 "ASTGRC.nw"
Status visit(StatementList &);
#line 979 "ASTGRC.nw"
Status visit(Loop &);
#line 1028 "ASTGRC.nw"
Status visit(Repeat &);
#line 1191 "ASTGRC.nw"
Status visit(Suspend &);
#line 1391 "ASTGRC.nw"
Status visit(Abort &);
#line 1476 "ASTGRC.nw"
Status visit(ParallelStatementList &);
#line 1735 "ASTGRC.nw"
Status visit(Trap &);
#line 1804 "ASTGRC.nw"
Status visit(Signal &);
#line 1843 "ASTGRC.nw"
Status visit(Var &);
#line 1859 "ASTGRC.nw"
Status visit(Exec &) { return Status(); }
#line 1873 "ASTGRC.nw"
Status visit(ProcedureCall &) { return Status(); }
#line 686 "ASTGRC.nw"
};
#line 2102 "ASTGRC.nw"
  
#line 647 "ASTGRC.nw"
class SelTree : public Visitor {
protected:
  GrcSynth &environment;
public:
  SelTree(GrcSynth &e): environment(e) {}

  STNode *synthesize(ASTNode *n) {
    assert(n);
    STNode *result = dynamic_cast<STNode*>(n->welcome(*this).n);
    assert(result);
    return result;
  }

  void setNode(const ASTNode &, STNode *);
    
  
#line 702 "ASTGRC.nw"
Status visit(Pause &);
#line 749 "ASTGRC.nw"
Status visit(Exit &s) {
  return Status(new STref());
}
#line 774 "ASTGRC.nw"
Status visit(Emit &) {
  return Status(new STref());
}
#line 799 "ASTGRC.nw"
Status visit(Assign &s) {
  return Status(new STref());
}
#line 822 "ASTGRC.nw"
Status visit(IfThenElse &);
#line 881 "ASTGRC.nw"
Status visit(StatementList &s);
#line 953 "ASTGRC.nw"
Status visit(Loop &s);
#line 1000 "ASTGRC.nw"
Status visit(Repeat &);
#line 1183 "ASTGRC.nw"
Status visit(Suspend &);
#line 1383 "ASTGRC.nw"
Status visit(Abort &);
#line 1399 "ASTGRC.nw"
Status visit(ParallelStatementList &);
#line 1727 "ASTGRC.nw"
Status visit(Trap &);
#line 1796 "ASTGRC.nw"
Status visit(Signal &);
#line 1835 "ASTGRC.nw"
Status visit(Var &);
#line 1851 "ASTGRC.nw"
Status visit(Exec &) { return Status(new STref()); }
#line 1865 "ASTGRC.nw"
Status visit(ProcedureCall &) { return Status(new STref()); }
#line 663 "ASTGRC.nw"
};
#line 2103 "ASTGRC.nw"
  
#line 469 "ASTGRC.nw"
struct GrcSynth {
  Module *module;
  CompletionCodes &code;

  Cloner clone;

  Context surface_context;
  Context depth_context;

  Surface surface;
  Depth depth;
  SelTree seltree;

  map<const ASTNode*, STNode*> ast2st;

  BuiltinTypeSymbol *integer_type;
  BuiltinTypeSymbol *boolean_type; 

  
#line 495 "ASTGRC.nw"
GrcSynth(Module *, CompletionCodes &);
#line 527 "ASTGRC.nw"
GRCgraph *synthesize()
{
  assert(module->body);

  // Set up initial and terminal states in the selection tree

  STexcl *stroot = new STexcl();
  STleaf *boot = new STleaf();
  STleaf *finished = new STleaf();
  finished->setfinal();

  // Set up the root of the GRC

  EnterGRC *engrc = new EnterGRC();
  ExitGRC *exgrc = new ExitGRC();

  Enter *enfinished = new Enter(finished);
  Switch *top_switch = new Switch(stroot);

  *engrc >> exgrc >> top_switch;
  *enfinished >> exgrc;

  enfinished->st = finished;

  Terminate *term0 = new Terminate(0, 0);
  *term0 >> enfinished;
  Terminate *term1 = new Terminate(1, 0);
  *term1 >> exgrc;

  // Set up the context for the surface and the depth: point to term0 and 1

  surface_context(0) = depth_context(0) = term0;
  surface_context(1) = depth_context(1) = term1;

  // Build the selection tree and create the selection tree root

  STNode *synt_seltree = seltree.synthesize(module->body);
  *stroot >> finished >> synt_seltree >> boot;

  // Build the surface and the depth and attach them to the top switch

  GRCNode *synt_surface = surface.synthesize(module->body);
  GRCNode *synt_depth = depth.synthesize(module->body);
  *top_switch >> enfinished >> synt_depth >> synt_surface;

  GRCgraph *result = new GRCgraph(stroot, engrc);

  return result;
}
#line 488 "ASTGRC.nw"
};

#line 2105 "ASTGRC.nw"
  
#line 1883 "ASTGRC.nw"
class Dependencies : public Visitor {
protected:
  set<GRCNode *> visited;
  GRCNode *current;

  struct SignalNodes {
    set<GRCNode *> writers;
    set<GRCNode *> readers;
  };

  map<SignalSymbol *, SignalNodes> dependencies;

  
#line 1949 "ASTGRC.nw"
void dfs(GRCNode *);
#line 1972 "ASTGRC.nw"
Status visit(Action &);
#line 1980 "ASTGRC.nw"
Status visit(DefineSignal &d) {
  dependencies[d.signal].writers.insert(current);
  return Status();
}
#line 1991 "ASTGRC.nw"
Status visit(Test &t) { t.predicate->welcome(*this); return Status(); }
#line 1997 "ASTGRC.nw"
Status visit(LoadSignalExpression &e) {
  dependencies[e.signal].readers.insert(current);
  return Status();
}

Status visit(LoadSignalValueExpression &e) {
  dependencies[e.signal].readers.insert(current);
  return Status();
}

Status visit(BinaryOp &e) {
  e.source1->welcome(*this);
  e.source2->welcome(*this);
  return Status();
}

Status visit(UnaryOp &e) {
  e.source->welcome(*this);
  return Status();
}

Status visit(CheckCounter &e) {
  e.counter->predicate->welcome(*this);
  return Status();
}

Status visit(Delay &d) {
  d.predicate->welcome(*this);
  return Status();
}
#line 2032 "ASTGRC.nw"
Status visit(Literal &) { return Status(); }
Status visit(LoadVariableExpression &) { return Status(); }
Status visit(FunctionCall &) { return Status(); }
#line 2056 "ASTGRC.nw"
Status visit(Sync &);
#line 2064 "ASTGRC.nw"
Status visit(EnterGRC &) { return Status(); }
Status visit(ExitGRC &) { return Status(); }
Status visit(Nop &) { return Status(); }
Status visit(Switch &) { return Status(); }
Status visit(STSuspend &) { return Status(); }
Status visit(Fork &) { return Status(); }
Status visit(Terminate &) { return Status(); }
Status visit(Enter &) { return Status(); }
#line 1896 "ASTGRC.nw"
  Dependencies() {}
  virtual ~Dependencies() {}
public:
  static void compute(GRCNode *);
};
#line 2106 "ASTGRC.nw"
}
#endif
